.myDashAce_js_metadata <- function() {
deps_metadata <- list(`my_dash_ace` = structure(list(name = "my_dash_ace",
version = "0.0.7", src = list(href = NULL,
file = "deps"), meta = NULL,
script = 'my_dash_ace.min.js',
stylesheet = NULL, head = NULL, attachment = NULL, package = "myDashAce",
all_files = FALSE), class = "html_dependency"),
`my_dash_ace` = structure(list(name = "my_dash_ace",
version = "0.0.7", src = list(href = NULL,
file = "deps"), meta = NULL,
script = 'my_dash_ace.min.js.map',
stylesheet = NULL, head = NULL, attachment = NULL, package = "myDashAce",
all_files = FALSE), class = "html_dependency"))
return(deps_metadata)
}
